/*
 * Source code for the Otter Print Shooter
 * By Hoong Chin Ng
 */
#include  <msp430g2553.h>

short int i, num, count_time, PIR_SIG;
short int clock_tick (void);

void main(void)
{
// configure clock
	DCOCTL = CALDCO_1MHZ; // 1MHz DCO
	BCSCTL1 = CALBC1_1MHZ; // MCLK
	BCSCTL2 |= 0x00;
	BCSCTL3 |= 0x24; // 12kHz VLO aka ACLK
	IFG1 = 0x00; // IFG1 &= ~OFIFG;
	IE1 = 0x00;

	WDTCTL = WDTPW + WDTHOLD;	// Stop watchdog timer

// configure Port1
	P1DIR = 0x0F;	// P1.4,5,6,7 input, else output
	P1OUT = 0x00;
	P1SEL = 0x00;
	P1SEL2 = 0x00;
	P1REN = 0x00;
	P1IE |= 0xF0; 	// P1.3,4,5,6,7 interrupt enable
	P1IES = 0x00;	// positive trigger
	P1IFG = 0x00;	// IFG cleared

// configure Port2
	P2DIR = ~BIT0 & ~BIT5; // P2.0 & P2.5 input, else output
	P2OUT = 0x00;
	P2SEL = BIT1;	// P2.1 PWM output
	P2SEL2 = 0x00;
	P2REN |= BIT0 + BIT5;	// enable pull up/down resistors
	P2IE |= BIT0 + BIT5;	// P2.0 & P2.5 interrupt enable
	P2IES = ~BIT0 & ~BIT5;	// positive trigger
	P2IFG = 0x00;

// configure timer0
	TA0CTL = 0x0100;
	TA0CCTL0 |= CCIE;
	TA0CCR0 = 2999;	// 0.25s interval

// configure timer1
	TA1CTL = 0x0200;
	TA1CCTL1 |= OUTMOD_7;
	TA1CCR0 = 9999;	// 10ms period
	TA1CCR1 = 500; // 5% duty cycle

	num = 0;
	count_time = 0;
	PIR_SIG = 0;

	while(1)
	{
		if (PIR_SIG == 1)
		{
			P2IE &= ~BIT0 & ~BIT5; // temporary disable port 2 interrupt
			PIR_SIG = 0;

			TA0CTL |= MC_1; // start timer0
			TA1CTL |= MC_1; // start timer1
			P2SEL |= BIT1;

			while (count_time < 16 && num == 0) // count until 4 sec
			{
				while (clock_tick () != 1) {}
					TA1CCR1 += 500; // increase duty cycle by 5%
			}

			P2SEL &= ~BIT1;
			TA0CTL &= ~(MC1 + MC0); // stop timer0
			TA1CTL &= ~(MC1 + MC0); // stop timer1
			TA1CCR1 = 500;	// reset duty cycle to 5%
			num = 0;
			count_time = 0;
// wait for 5 sec before next motion detection
			for (i = 0 ; i < 20000 ; i++)
				_delay_cycles(50);
			for (i = 0 ; i < 20000 ; i++)
				_delay_cycles(50);
			for (i = 0 ; i < 20000 ; i++)
				_delay_cycles(50);
			for (i = 0 ; i < 20000 ; i++)
				_delay_cycles(50);

			P2IE |= BIT0 + BIT5;	// enable back port 2 interrupt
		}

		_BIS_SR(LPM3_bits + GIE);	// Enter LPM3 with interrupt enabled
	}
}

// port 1 interrupt
#pragma vector = PORT1_VECTOR
__interrupt void FSR (void)
{
	num = 0;
	P2SEL &= ~BIT1;
	P2OUT |= BIT1;
// capture max of 3 images when hold down
	while ( ((P1IN & 0xF0)!=0x00) && (num < 3) )
	{

		P1OUT |= BIT0;	// camera on
		_delay_cycles(100000);	// Delay 0.1s

		P1OUT &= ~BIT0; // camera off
		_delay_cycles(900000);	// Delay 0.9s
		num ++;
	}

	P2OUT &= ~BIT1;
	P1IFG = 0x00;

	_BIC_SR_IRQ(LPM3_bits);	// Clear LPM3 bits from SR
}

// port2 interrupt
#pragma vector = PORT2_VECTOR
__interrupt void PIR (void)
{
	PIR_SIG = 1;
	P2IFG = 0x00;
	_BIC_SR_IRQ(LPM3_bits);	// Clear LPM3 bits from SR
}

short int clock_tick (void)
{
	short int temp;
	temp = count_time;
	while (temp == count_time){}

	return 1;
}

// 0.25s count
#pragma vector = TIMER0_A0_VECTOR
__interrupt void Timer0_A (void)
{
	count_time ++;
}
